==> xl-react-components-0.21.15.tgz: package/app/xl-react-dip/xl-react-dip-map-string-string/xl-react-dip-map-string-string.js <==
// eslint-disable-line max-lines
import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import $ from 'jquery';
import {isDisabled} from '../../utils/components-utils';
import {XlReactDipContainerProps} from '../xl-react-dip-props';
import RU from '../../utils/ramda-utils';
import {computeNewStateOnNewKeyBlur, isKeyUnique} from '../input/input-service';
import {XlReactDipMapStringStringComponent} from './xl-react-dip-map-string-string-component';
import * as service from './xl-react-dip-map-string-string-service';
import {COLUMNS, getUpdatedCellState, NEW_VALUE_CELL_KEY} from './map-table-utils';

const delayTime = 200;

const resetAllErrors = {
    duplicateKey: undefined,
    emptyKey: undefined,
    hasNewKeyError: undefined
};

export class XlReactDipMapStringString extends Component {
    static propTypes = {
        model: PropTypes.object,
        pristine: PropTypes.bool,
        showNewKeyValueRow: PropTypes.bool,
        valid: PropTypes.bool,
        ...XlReactDipContainerProps
    };

    static defaultProps = {
        displayMode: 'edit',
        metadata: {},
        validators: []
    };

    static getDerivedStateFromProps(nextProps, previousState) {
        const pristine = nextProps.pristine;
        if (pristine && !previousState.pristine) {
            const passwordsCanBeRevealed = R.pipe(
                R.keys,
                R.reduce((acc, key) => R.assoc(key, !nextProps.model[key], acc), previousState.passwordsCanBeRevealed)
            )(nextProps.model);
            return {
                passwordsCanBeRevealed,
                passwordsRevealed: {[NEW_VALUE_CELL_KEY]: false},
                pristine
            };
        }
        return {pristine};
    }

    constructor(props) {
        super(props);
        this.state = {
            focusedValue: null,
            modelKeys: service.getModelKeys(props.model, true),
            newKey: '',
            newValue: '',
            passwordsCanBeRevealed: {[NEW_VALUE_CELL_KEY]: true},
            passwordsRevealed: {[NEW_VALUE_CELL_KEY]: false},
            pristine: props.pristine,
            searchWord: '',
            showNewKeyValueRow: false
        };
        RU.invoke('register', [this.api], props);
    }

    componentWillUnmount() {
        RU.invoke('unregister', [], this.props);
    }

    handleCleaningSearchWord = () => {
        this.setState({searchWord: ''});
    };

    handleKeyBlur = (changedKey, oldKey, input) => {
        const {modelKeys} = this.state;
        const {model} = this.props;
        if (!R.has(oldKey, model)) {
            return;
        }

        const trimmedChangedKey = R.trim(changedKey);
        const hasKeyChanged = !R.equals(trimmedChangedKey, oldKey);

        const handleChangedKey = () => {
            if (!trimmedChangedKey) {
                this.setState({emptyKey: true});
                RU.invoke('focus', [], input);
            } else if (!isKeyUnique(trimmedChangedKey, oldKey)(model)) {
                this.setState({duplicateKey: trimmedChangedKey});
                RU.invoke('focus', [], input);
            } else {
                const updatedModel = service.getModelWithUpdatedKey(model, oldKey, trimmedChangedKey);
                const updatedModelKeys = service.getModelKeysWithUpdatedKey(modelKeys, oldKey, trimmedChangedKey);
                this.setState({modelKeys: updatedModelKeys});

                const focusToInputField = () => {
                    if (this.state.hasNewKeyError) {
                        this.delayedKeyOnInputFocus();
                    } else {
                        this.focusKeyInput(trimmedChangedKey);
                    }
                };
                this.applyNewState(updatedModel, focusToInputField);
            }
        };

        if (!hasKeyChanged) {
            if (this.state.hasNewKeyError) {
                this.delayedKeyOnInputFocus();
            } else {
                this.setState(resetAllErrors);
            }
        } else {
            handleChangedKey();
        }
    };

    handleRemoveKey = (key) => {
        const model = R.dissoc(key, this.props.model);
        const modelKeys = R.without([key], this.state.modelKeys);
        this.setState({modelKeys});
        this.applyNewState(model, this.focusOnInputKeyWithError);
    };

    handleSearchChanged = (searchWord) => {
        this.setState({searchWord});
    };

    handleSortChange = ({sortBy, sortDirection}) => {
        this.setState({
            cellState: getUpdatedCellState(this.props.model, this.state.cellState, {sortBy, sortDirection}),
            sorting: {sortBy, sortDirection}
        });
    };

    handleValueChange = (key, newValue) => {
        if (R.has(key, this.props.model)) {
            this.applyNewState(R.assoc(key, newValue, this.props.model), this.focusOnInputKeyWithError);
        }

        if (newValue.length === 0 && R.isNil(this.state.passwordsRevealed[key])) {
            this.handleOnPasswordPaste(key)();
        }
    };

    handleRevealPassword = (key) => () => {
        this.setState(R.evolve({passwordsRevealed: {[key]: R.not}}));
    };

    handleOnPasswordPaste = (key) => () => {
        const newState = R.pipe(
            R.assocPath(['passwordsRevealed', key], false),
            R.assocPath(['passwordsCanBeRevealed', key], true)
        )(this.state);
        this.setState(newState);
    };

    onCloseButton = () => {
        this.setState({
            ...resetAllErrors,
            newKey: '',
            newValue: '',
            showNewKeyValueRow: false
        });
    };

    onAddRow = () => setTimeout(() => {
        if (!this.state.hasNewKeyError) {
            this.setState({
                cellState: {
                    column: COLUMNS.key,
                    row: 0,
                    value: ''
                },
                searchWord: '',
                showNewKeyValueRow: true
            }, () => {
                this.tableRef.scrollToRow(0);
                this.delayedKeyOnInputFocus();
            });
        }
    }, delayTime);

    onNewKeyBlurDelayedHandler = () => {
        const newState = computeNewStateOnNewKeyBlur(this.state, this.props.model);
        const applyModelIfPossible = () => {
            const {newKey, focusedValue} = this.state;

            if (focusedValue !== NEW_VALUE_CELL_KEY && R.trim(newKey)) {
                this.addModelKey();
            }
        };
        this.setState({...newState}, applyModelIfPossible);
    };

    onNewKeyBlur = () => setTimeout(this.onNewKeyBlurDelayedHandler, delayTime);

    onNewKeyChange = (event) => {
        this.setState({newKey: event.target.value});
    };

    onNewKeyFocus = () => {
        this.setState(resetAllErrors);
    };

    onNewValueBlur = () => {
        const {hasNewKeyError, newKey} = this.state;

        this.setState({
            focusedValue: null,
            passwordsCanBeRevealed: {
                ...this.state.passwordsCanBeRevealed,
                [NEW_VALUE_CELL_KEY]: true,
                [newKey]: this.state.passwordsCanBeRevealed[NEW_VALUE_CELL_KEY]
            },
            passwordsRevealed: {
                ...this.state.passwordsRevealed,
                [NEW_VALUE_CELL_KEY]: false,
                [newKey]: this.state.passwordsRevealed[NEW_VALUE_CELL_KEY]
            }
        });

        if (hasNewKeyError) {
            this.delayedKeyOnInputFocus();
        } else if (R.trim(newKey)) {
            this.addModelKey();
        }
    };

    onValueFocus = (key) => () => {
        this.setState({focusedValue: key});
    };

    onNewValueKeyDown = ({target: {value}}) => {
        this.setState({newValue: value});
    };

    delayedKeyOnInputFocus = () => setTimeout(() => {
        this.inputKeyField.focus();
    }, delayTime);

    addModelKey = () => {
        const {newKey, newValue, modelKeys} = this.state;
        const {model} = this.props;
        const keyIsUnique = isKeyUnique(newKey)(model);
        if (keyIsUnique) {
            this.setState({
                modelKeys: [newKey, ...modelKeys],
                newKey: '',
                newValue: '',
                showNewKeyValueRow: false
            });
            const focusOnField = () => {
                this.focusKeyInput(newKey);
            };
            this.applyNewState({...model, [newKey]: newValue}, focusOnField);
        } else {
            this.setState({
                duplicateKey: newKey,
                emptyKey: undefined,
                hasNewKeyError: true
            });
        }
    };

    focusKeyInput = (key) => {
        const tableInputs = R.values($(this.tableWrapperRef)
            .find('.key-div input'));
        const mapKeyInputField = R.find(R.propSatisfies(R.equals(key), 'defaultValue'), tableInputs);
        if (mapKeyInputField) {
            mapKeyInputField.focus();
        }
    };

    focusOnInputKeyWithError = () => {
        if (this.state.hasNewKeyError) {
            this.delayedKeyOnInputFocus();
        }
    };

    api = {
        focus: () => {
            this.setState({showNewKeyValueRow: true}, () => {
                this.delayedKeyOnInputFocus();
            });
        }
    };

    applyNewState = (model, callback = R.F) => {
        this.props.onModelChange(model);
        this.setState({emptyKey: undefined}, callback);
    };

    assignRef = (input) => {
        this.inputKeyField = input;
    };

    updateCellState = (cellState) => {
        this.setState({cellState});
    };

    assignTableRef = (table) => {
        this.tableRef = table;
    };

    assignTableWrapperRef = (tableWrapper) => {
        this.tableWrapperRef = tableWrapper;
    };

    render() {
        const {
            displayMode,
            message,
            metadata,
            model,
            onValidityChange,
            pristine,
            valid
        } = this.props;
        const {
            cellState,
            duplicateKey,
            emptyKey,
            focusedValue,
            modelKeys,
            newKey,
            newValue,
            passwordsCanBeRevealed,
            passwordsRevealed,
            showNewKeyValueRow,
            searchWord,
            sorting
        } = this.state;
        const {autoHide, description, label, password, required} = metadata;
        const disabled = isDisabled(this.props.disabled, displayMode);

        return <XlReactDipMapStringStringComponent {...({
            assignRef: this.assignRef,
            assignTableRef: this.assignTableRef,
            assignTableWrapperRef: this.assignTableWrapperRef,
            autoHide,
            cellState,
            description,
            disabled,
            duplicateKey,
            emptyKey,
            encrypted: password,
            focusedValue,
            handleAddRow: this.onAddRow,
            handleCloseButton: this.onCloseButton,
            handleFocus: this.onValueFocus,
            handleNewKeyBlur: this.onNewKeyBlur,
            handleNewKeyChange: this.onNewKeyChange,
            handleNewKeyFocus: this.onNewKeyFocus,
            handleNewValueBlur: this.onNewValueBlur,
            handleNewValueKeyDown: this.onNewValueKeyDown,
            handleOnPasswordPaste: this.handleOnPasswordPaste,
            handleRevealPassword: this.handleRevealPassword,
            handleSortChange: this.handleSortChange,
            label,
            message,
            model,
            modelKeys,
            newKey,
            newValue,
            onCleaningSearchWord: this.handleCleaningSearchWord,
            onKeyBlur: this.handleKeyBlur,
            onRemoveKey: this.handleRemoveKey,
            onSearchChanged: this.handleSearchChanged,
            onValidityChange,
            onValueChange: this.handleValueChange,
            passwordsCanBeRevealed,
            passwordsRevealed,
            pristine,
            required,
            searchWord,
            showNewKeyValueRow,
            sorting,
            updateCellState: this.updateCellState,
            valid
        })}
        />;
    }
}

// eslint-enable-line max-lines

==> xl-react-components-0.21.16.tgz: package/app/xl-react-dip/xl-react-dip-map-string-string/xl-react-dip-map-string-string.js <==
// eslint-disable-line max-lines
import React, {Component} from 'react';
import PropTypes from 'prop-types';
import R from 'ramda';
import $ from 'jquery';
import {isDisabled} from '../../utils/components-utils';
import {XlReactDipContainerProps} from '../xl-react-dip-props';
import RU from '../../utils/ramda-utils';
import {computeNewStateOnNewKeyBlur, isKeyUnique} from '../input/input-service';
import {XlReactDipMapStringStringComponent} from './xl-react-dip-map-string-string-component';
import * as service from './xl-react-dip-map-string-string-service';
import {COLUMNS, getUpdatedCellState, NEW_VALUE_CELL_KEY} from './map-table-utils';

const delayTime = 200;

const resetAllErrors = {
    duplicateKey: undefined,
    emptyKey: undefined,
    hasNewKeyError: undefined
};

export class XlReactDipMapStringString extends Component {
    static propTypes = {
        model: PropTypes.object,
        pristine: PropTypes.bool,
        showNewKeyValueRow: PropTypes.bool,
        valid: PropTypes.bool,
        ...XlReactDipContainerProps
    };

    static defaultProps = {
        displayMode: 'edit',
        metadata: {},
        validators: []
    };

    static getDerivedStateFromProps(nextProps, previousState) {
        const pristine = nextProps.pristine;
        if (pristine && !previousState.pristine) {
            const passwordsCanBeRevealed = R.pipe(
                R.keys,
                R.reduce((acc, key) => R.assoc(key, !nextProps.model[key], acc), previousState.passwordsCanBeRevealed)
            )(nextProps.model);
            return {
                passwordsCanBeRevealed,
                passwordsRevealed: {[NEW_VALUE_CELL_KEY]: false},
                pristine
            };
        }
        return {pristine};
    }

    constructor(props) {
        super(props);
        this.state = {
            editedValue: '',
            focusedValue: null,
            modelKeys: service.getModelKeys(props.model, true),
            newKey: '',
            newValue: '',
            passwordsCanBeRevealed: {[NEW_VALUE_CELL_KEY]: true},
            passwordsRevealed: {[NEW_VALUE_CELL_KEY]: false},
            pristine: props.pristine,
            searchWord: '',
            showNewKeyValueRow: false
        };
        RU.invoke('register', [this.api], props);
    }

    componentWillUnmount() {
        RU.invoke('unregister', [], this.props);
    }

    handleCleaningSearchWord = () => {
        this.setState({searchWord: ''});
    };

    handleKeyBlur = (changedKey, oldKey, input) => {
        const {modelKeys} = this.state;
        const {model} = this.props;
        if (!R.has(oldKey, model)) {
            return;
        }

        const trimmedChangedKey = R.trim(changedKey);
        const hasKeyChanged = !R.equals(trimmedChangedKey, oldKey);

        const handleChangedKey = () => {
            if (!trimmedChangedKey) {
                this.setState({emptyKey: true});
                RU.invoke('focus', [], input);
            } else if (!isKeyUnique(trimmedChangedKey, oldKey)(model)) {
                this.setState({duplicateKey: trimmedChangedKey});
                RU.invoke('focus', [], input);
            } else {
                const updatedModel = service.getModelWithUpdatedKey(model, oldKey, trimmedChangedKey);
                const updatedModelKeys = service.getModelKeysWithUpdatedKey(modelKeys, oldKey, trimmedChangedKey);
                this.setState({modelKeys: updatedModelKeys});

                const focusToInputField = () => {
                    if (this.state.hasNewKeyError) {
                        this.delayedKeyOnInputFocus();
                    } else {
                        this.focusKeyInput(trimmedChangedKey);
                    }
                };
                this.applyNewState(updatedModel, focusToInputField);
            }
        };

        if (!hasKeyChanged) {
            if (this.state.hasNewKeyError) {
                this.delayedKeyOnInputFocus();
            } else {
                this.setState(resetAllErrors);
            }
        } else {
            handleChangedKey();
        }
    };

    handleRemoveKey = (key) => {
        const model = R.dissoc(key, this.props.model);
        const modelKeys = R.without([key], this.state.modelKeys);
        this.setState({modelKeys});
        this.applyNewState(model, this.focusOnInputKeyWithError);
    };

    handleSearchChanged = (searchWord) => {
        this.setState({searchWord});
    };

    handleSortChange = ({sortBy, sortDirection}) => {
        this.setState({
            cellState: getUpdatedCellState(this.props.model, this.state.cellState, {sortBy, sortDirection}),
            sorting: {sortBy, sortDirection}
        });
    };

    handleValueBlur = () => {
        const key = this.state.focusedValue;
        const newValue = this.state.editedValue;
        if (R.has(key, this.props.model)) {
            this.applyNewState(R.assoc(key, newValue, this.props.model), this.focusOnInputKeyWithError);
        }
        this.checkChangedPassword(newValue, key);
    }

    handleValueChange = (newValue) => {
        this.checkChangedPassword(newValue, this.state.focusedValue);
        this.setState({editedValue: newValue});
    };

    handleRevealPassword = (key) => () => {
        this.setState(R.evolve({passwordsRevealed: {[key]: R.not}}));
    };

    handleOnPasswordPaste = (key) => () => {
        const newState = R.pipe(
            R.assocPath(['passwordsRevealed', key], false),
            R.assocPath(['passwordsCanBeRevealed', key], true)
        )(this.state);
        this.setState(newState);
    };

    onCloseButton = () => {
        this.setState({
            ...resetAllErrors,
            newKey: '',
            newValue: '',
            showNewKeyValueRow: false
        });
    };

    onAddRow = () => setTimeout(() => {
        if (!this.state.hasNewKeyError) {
            this.setState({
                cellState: {
                    column: COLUMNS.key,
                    row: 0,
                    value: ''
                },
                searchWord: '',
                showNewKeyValueRow: true
            }, () => {
                this.tableRef.scrollToRow(0);
                this.delayedKeyOnInputFocus();
            });
        }
    }, delayTime);

    onNewKeyBlurDelayedHandler = () => {
        const newState = computeNewStateOnNewKeyBlur(this.state, this.props.model);
        const applyModelIfPossible = () => {
            const {newKey, focusedValue} = this.state;

            if (focusedValue !== NEW_VALUE_CELL_KEY && R.trim(newKey)) {
                this.addModelKey();
            }
        };
        this.setState({...newState}, applyModelIfPossible);
    };

    onNewKeyBlur = () => setTimeout(this.onNewKeyBlurDelayedHandler, delayTime);

    onNewKeyChange = (event) => {
        this.setState({newKey: event.target.value});
    };

    onNewKeyFocus = () => {
        this.setState(resetAllErrors);
    };

    onNewValueBlur = () => {
        const {hasNewKeyError, newKey} = this.state;

        this.setState({
            focusedValue: null,
            passwordsCanBeRevealed: {
                ...this.state.passwordsCanBeRevealed,
                [NEW_VALUE_CELL_KEY]: true,
                [newKey]: this.state.passwordsCanBeRevealed[NEW_VALUE_CELL_KEY]
            },
            passwordsRevealed: {
                ...this.state.passwordsRevealed,
                [NEW_VALUE_CELL_KEY]: false,
                [newKey]: this.state.passwordsRevealed[NEW_VALUE_CELL_KEY]
            }
        });

        if (hasNewKeyError) {
            this.delayedKeyOnInputFocus();
        } else if (R.trim(newKey)) {
            this.addModelKey();
        }
    };

    onValueFocus = (key) => () => {
        this.setState({
            editedValue: this.props.model[key],
            focusedValue: key
        });
    };

    onNewValueKeyDown = ({target: {value}}) => {
        this.setState({newValue: value});
    };

    checkChangedPassword = (newValue, key) => {
        if (newValue.length === 0 && R.isNil(this.state.passwordsRevealed[key])) {
            this.handleOnPasswordPaste(key)();
        }
    }

    delayedKeyOnInputFocus = () => setTimeout(() => {
        this.inputKeyField.focus();
    }, delayTime);

    addModelKey = () => {
        const {newKey, newValue, modelKeys} = this.state;
        const {model} = this.props;
        const keyIsUnique = isKeyUnique(newKey)(model);
        if (keyIsUnique) {
            this.setState({
                modelKeys: [newKey, ...modelKeys],
                newKey: '',
                newValue: '',
                showNewKeyValueRow: false
            });
            const focusOnField = () => {
                this.focusKeyInput(newKey);
            };
            this.applyNewState({...model, [newKey]: newValue}, focusOnField);
        } else {
            this.setState({
                duplicateKey: newKey,
                emptyKey: undefined,
                hasNewKeyError: true
            });
        }
    };

    focusKeyInput = (key) => {
        const tableInputs = R.values($(this.tableWrapperRef)
            .find('.key-div input'));
        const mapKeyInputField = R.find(R.propSatisfies(R.equals(key), 'defaultValue'), tableInputs);
        if (mapKeyInputField) {
            mapKeyInputField.focus();
        }
    };

    focusOnInputKeyWithError = () => {
        if (this.state.hasNewKeyError) {
            this.delayedKeyOnInputFocus();
        }
    };

    api = {
        focus: () => {
            this.setState({showNewKeyValueRow: true}, () => {
                this.delayedKeyOnInputFocus();
            });
        }
    };

    applyNewState = (model, callback = R.F) => {
        this.props.onModelChange(model);
        this.setState({emptyKey: undefined}, callback);
    };

    assignRef = (input) => {
        this.inputKeyField = input;
    };

    updateCellState = (cellState) => {
        this.setState({cellState});
    };

    assignTableRef = (table) => {
        this.tableRef = table;
    };

    assignTableWrapperRef = (tableWrapper) => {
        this.tableWrapperRef = tableWrapper;
    };

    render() {
        const {
            displayMode,
            message,
            metadata,
            model,
            onValidityChange,
            pristine,
            valid
        } = this.props;
        const {
            cellState,
            duplicateKey,
            editedValue,
            emptyKey,
            focusedValue,
            modelKeys,
            newKey,
            newValue,
            passwordsCanBeRevealed,
            passwordsRevealed,
            showNewKeyValueRow,
            searchWord,
            sorting
        } = this.state;
        const {autoHide, description, label, password, required} = metadata;
        const disabled = isDisabled(this.props.disabled, displayMode);

        return <XlReactDipMapStringStringComponent {...({
            assignRef: this.assignRef,
            assignTableRef: this.assignTableRef,
            assignTableWrapperRef: this.assignTableWrapperRef,
            autoHide,
            cellState,
            description,
            disabled,
            duplicateKey,
            editedValue,
            emptyKey,
            encrypted: password,
            focusedValue,
            handleAddRow: this.onAddRow,
            handleCloseButton: this.onCloseButton,
            handleFocus: this.onValueFocus,
            handleNewKeyBlur: this.onNewKeyBlur,
            handleNewKeyChange: this.onNewKeyChange,
            handleNewKeyFocus: this.onNewKeyFocus,
            handleNewValueBlur: this.onNewValueBlur,
            handleNewValueKeyDown: this.onNewValueKeyDown,
            handleOnPasswordPaste: this.handleOnPasswordPaste,
            handleRevealPassword: this.handleRevealPassword,
            handleSortChange: this.handleSortChange,
            label,
            message,
            model,
            modelKeys,
            newKey,
            newValue,
            onCleaningSearchWord: this.handleCleaningSearchWord,
            onKeyBlur: this.handleKeyBlur,
            onRemoveKey: this.handleRemoveKey,
            onSearchChanged: this.handleSearchChanged,
            onValidityChange,
            onValueBlur: this.handleValueBlur,
            onValueChange: this.handleValueChange,
            passwordsCanBeRevealed,
            passwordsRevealed,
            pristine,
            required,
            searchWord,
            showNewKeyValueRow,
            sorting,
            updateCellState: this.updateCellState,
            valid
        })}
        />;
    }
}

// eslint-enable-line max-lines
